package sun.security.krb5.internal;

import java.net.Inet6Address;
import java.net.Inet4Address;
import java.util.ArrayList;
import sun.security.krb5.internal.ccache.CCacheOutputStream;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import sun.security.krb5.Asn1Exception;
import java.util.Vector;
import sun.security.util.DerValue;
import sun.security.krb5.KrbException;
import java.net.InetAddress;
import sun.security.krb5.PrincipalName;
import java.net.UnknownHostException;
import java.io.IOException;

public class HostAddresses implements Cloneable
{
    private static boolean DEBUG;
    private HostAddress[] addresses;
    private volatile int hashCode;
    
    public HostAddresses(HostAddress[] array) throws IOException {
        this.addresses = null;
        this.hashCode = 0;
        if (array != null) {
            this.addresses = new HostAddress[array.length];
            int i = 0;
            while (i < array.length) {
                if (array[i] == null) {
                    throw new IOException("Cannot create a HostAddress");
                }
                this.addresses[i] = (HostAddress)array[i].clone();
                ++i;
            }
        }
    }
    
    public HostAddresses() throws UnknownHostException {
        this.addresses = null;
        this.hashCode = 0;
        (this.addresses = new HostAddress[1])[0] = new HostAddress();
    }
    
    private HostAddresses(int n) {
        this.addresses = null;
        this.hashCode = 0;
    }
    
    public HostAddresses(PrincipalName principalName) throws UnknownHostException, KrbException {
        this.addresses = null;
        this.hashCode = 0;
        String[] nameStrings = principalName.getNameStrings();
        if (principalName.getNameType() != 3 || nameStrings.length < 2) {
            throw new KrbException(60, "Bad name");
        }
        InetAddress[] allByName = InetAddress.getAllByName(nameStrings[1]);
        HostAddress[] addresses = new HostAddress[allByName.length];
        for (int i = 0; i < allByName.length; ++i) {
            addresses[i] = new HostAddress(allByName[i]);
        }
        this.addresses = addresses;
        return;
    }
    
    public Object clone() {
        HostAddresses hostAddresses = new HostAddresses(0);
        if (this.addresses != null) {
            hostAddresses.addresses = new HostAddress[this.addresses.length];
            for (int i = 0; i < this.addresses.length; ++i) {
                hostAddresses.addresses[i] = (HostAddress)this.addresses[i].clone();
            }
            return hostAddresses;
        }
        return hostAddresses;
    }
    
    public boolean inList(HostAddress hostAddress) {
        if (this.addresses != null) {
            int i = 0;
            while (i < this.addresses.length) {
                if (this.addresses[i].equals(hostAddress)) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        if (this.hashCode == 0) {
            int hashCode = 17;
            if (this.addresses != null) {
                for (int i = 0; i < this.addresses.length; ++i) {
                    hashCode = 37 * hashCode + this.addresses[i].hashCode();
                }
            }
            this.hashCode = hashCode;
        }
        return this.hashCode;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof HostAddresses)) {
            return false;
        }
        HostAddresses hostAddresses = (HostAddresses)o;
        if ((this.addresses == null && hostAddresses.addresses != null) || (this.addresses != null && hostAddresses.addresses == null)) {
            return false;
        }
        if (this.addresses != null && hostAddresses.addresses != null) {
            if (this.addresses.length != hostAddresses.addresses.length) {
                return false;
            }
            int i = 0;
            while (i < this.addresses.length) {
                if (!this.addresses[i].equals(hostAddresses.addresses[i])) {
                    return false;
                }
                ++i;
            }
        }
        return true;
    }
    
    public HostAddresses(DerValue derValue) throws Asn1Exception, IOException {
        this.addresses = null;
        this.hashCode = 0;
        Vector<HostAddress> vector = new Vector();
        while (derValue.getData().available() > 0) {
            vector.addElement(new HostAddress(derValue.getData().getDerValue()));
        }
        if (vector.size() > 0) {
            vector.copyInto(this.addresses = new HostAddress[vector.size()]);
        }
    }
    
    public byte[] asn1Encode() throws Asn1Exception, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        DerOutputStream derOutputStream2 = new DerOutputStream();
        if (this.addresses != null && this.addresses.length > 0) {
            for (int i = 0; i < this.addresses.length; ++i) {
                derOutputStream.write(this.addresses[i].asn1Encode());
            }
        }
        derOutputStream2.write((byte)48, derOutputStream);
        return derOutputStream2.toByteArray();
    }
    
    public static HostAddresses parse(DerInputStream derInputStream, byte b, boolean b2) throws Asn1Exception, IOException {
        if (b2 && ((byte)derInputStream.peekByte() & 0x1F) != b) {
            return null;
        }
        DerValue derValue = derInputStream.getDerValue();
        if (b != (derValue.getTag() & 0x1F)) {
            throw new Asn1Exception(906);
        }
        return new HostAddresses(derValue.getData().getDerValue());
    }
    
    public void writeAddrs(CCacheOutputStream cCacheOutputStream) throws IOException {
        cCacheOutputStream.write32(this.addresses.length);
        for (int i = 0; i < this.addresses.length; ++i) {
            cCacheOutputStream.write16(this.addresses[i].addrType);
            cCacheOutputStream.write32(this.addresses[i].address.length);
            cCacheOutputStream.write(this.addresses[i].address, 0, this.addresses[i].address.length);
        }
    }
    
    public InetAddress[] getInetAddresses() {
        if (this.addresses == null || this.addresses.length == 0) {
            return null;
        }
        ArrayList list = new ArrayList(this.addresses.length);
        for (int i = 0; i < this.addresses.length; ++i) {
            try {
                if (this.addresses[i].addrType == 2 || this.addresses[i].addrType == 24) {
                    list.add(this.addresses[i].getInetAddress());
                }
            }
            catch (UnknownHostException ex) {
                return null;
            }
        }
        return (InetAddress[])list.toArray(new InetAddress[list.size()]);
    }
    
    public static HostAddresses getLocalAddresses() throws IOException {
        try {
            String hostName = InetAddress.getLocalHost().getHostName();
            InetAddress[] allByName = InetAddress.getAllByName(hostName);
            HostAddress[] array = new HostAddress[allByName.length];
            for (int i = 0; i < allByName.length; ++i) {
                array[i] = new HostAddress(allByName[i]);
            }
            if (HostAddresses.DEBUG) {
                System.out.println(">>> KrbKdcReq local addresses for " + hostName + " are: ");
                for (int j = 0; j < allByName.length; ++j) {
                    System.out.println("\n\t" + allByName[j]);
                    if ((allByName[j] instanceof Inet4Address)) {
                        System.out.println("IPv4 address");
                    }
                    if ((allByName[j] instanceof Inet6Address)) {
                        System.out.println("IPv6 address");
                    }
                }
            }
            return new HostAddresses(array);
        }
        catch (Exception ex) {
            throw new IOException(ex.toString());
        }
    }
    
    public HostAddresses(InetAddress[] array) {
        this.addresses = null;
        this.hashCode = 0;
        if (array == null) {
            this.addresses = null;
            return;
        }
        this.addresses = new HostAddress[array.length];
        for (int i = 0; i < array.length; ++i) {
            this.addresses[i] = new HostAddress(array[i]);
        }
        return;
    }
    
    static {
        HostAddresses.DEBUG = Krb5.DEBUG;
    }
}
